@@ -1,16 +1,21 @@
+from __future__ import annotations
+
 from abc import abstractmethod
 from collections import defaultdict
+from typing import TYPE_CHECKING, ClassVar
 from weakref import ref
 
 from observ import reactive, readonly
 
-from collagraph import render_slot
+if TYPE_CHECKING:  # pragma: no cover
+    from collagraph.fragment import ComponentFragment
+    from collagraph.renderers import Renderer
 
 
 class Component:
     """Abstract base class for components"""
 
-    __lookup_cache__ = defaultdict(dict)
+    __lookup_cache__: ClassVar = defaultdict(dict)
 
     def __init__(self, props=None, parent=None):
         self._props = readonly({} if props is None else props)
@@ -22,6 +27,12 @@ class Component:
         self._parent = ref(parent) if parent else None
         self._provided = {}
 
+        # Call init hook at end of __init__
+        self.init()
+
+    def init(self):
+        pass
+
     @property
     def props(self):
         """The incoming props of this component."""
@@ -49,7 +60,7 @@ class Component:
 
     @element.setter
     def element(self, value):
-        """Prevent overwriting the element attribute."""
+        """Prevent overwriting the element attributes."""
         raise RuntimeError("Not allowed to override element attribute")
 
     @property
@@ -65,12 +76,6 @@ class Component:
         """Prevent overwriting the parent attribute."""
         raise RuntimeError("Not allowed to override parent attribute")
 
-    def render_slot(self, name, props=None):
-        return render_slot(name, props, self._slots)
-
-    # Provide shortcut to render_slot method
-    s = render_slot
-
     def mounted(self):
         """Called after the component has been mounted.
 
@@ -102,8 +107,8 @@ class Component:
         pass
 
     @abstractmethod
-    def render():  # pragma: no cover
-        pass
+    def render(self, renderer: Renderer) -> ComponentFragment:  # pragma: no cover
+        raise NotImplementedError
 
     def provide(self, key: str, value):
         self._provided[key] = value
