@@ -1,15 +1,23 @@
 from abc import ABCMeta, abstractmethod
 from typing import Any, Callable, Optional
 
-from collagraph.types import EventLoopType
+from collagraph.constants import EventLoopType
 
 
 class Renderer(metaclass=ABCMeta):  # pragma: no cover
     """Abstract base class for renderers"""
 
     def preferred_event_loop_type(self) -> Optional[EventLoopType]:
+        """Indicate the preferred default event loop type"""
         return None
 
+    def register_asyncio(self) -> None:
+        """
+        Called by collagraph instance when registering the scheduler with
+        asyncio so that the renderer can perform any additional setup.
+        """
+        pass
+
     @abstractmethod
     def create_element(self, type: str) -> Any:
         """Create an element for the given type."""
@@ -60,14 +68,7 @@ class Renderer(metaclass=ABCMeta):  # pragma: no cover
         pass
 
 
-from .dict_renderer import DictRenderer  # noqa: I202
-
-try:
-    import js
-except ImportError:  # pragma: no cover
-    pass
-else:
-    from .dom_renderer import DomRenderer
+from .dict_renderer import DictRenderer
 
 try:
     import pygfx
@@ -81,4 +82,7 @@ try:
 except ImportError:  # pragma: no cover
     pass
 else:
-    from .pyside_renderer import PySideRenderer
+    from .pyside_renderer import PySideRenderer  # noqa: I001
+
+    # Need to import the objects in order to register their methods
+    from .pyside import objects
