@@ -1,8 +1,10 @@
 from PySide6.QtWidgets import QDialogButtonBox
 
+from ... import PySideRenderer
 from .widget import set_attribute as widget_set_attribute
 
 
+@PySideRenderer.register_insert(QDialogButtonBox)
 def insert(self, el, anchor=None):
     if hasattr(el, "role"):
         # Allow the role to be either an actual ButtonRole or a string
@@ -16,6 +18,7 @@ def insert(self, el, anchor=None):
     raise NotImplementedError
 
 
+@PySideRenderer.register_set_attr(QDialogButtonBox)
 def set_attribute(self, attr, value):
     if attr == "buttons":
         for flag in value:
