@@ -1,6 +1,9 @@
 from PySide6.QtGui import QStandardItemModel
 
+from ... import PySideRenderer
 
+
+@PySideRenderer.register_insert(QStandardItemModel)
 def insert(self, el, anchor=None):
     if isinstance(self, QStandardItemModel):
         if hasattr(el, "model_index"):
@@ -17,6 +20,7 @@ def insert(self, el, anchor=None):
         raise NotImplementedError(type(self).__name__)
 
 
+@PySideRenderer.register_remove(QStandardItemModel)
 def remove(self, el):
     index = self.indexFromItem(el)
     self.takeRow(index.row())
