@@ -1,6 +1,9 @@
-from PySide6.QtWidgets import QWidget
+from PySide6.QtWidgets import QSplitter, QWidget
+
+from ... import PySideRenderer
 
 
+@PySideRenderer.register_insert(QSplitter)
 def insert(self, el: QWidget, anchor: QWidget = None):
     if anchor is not None:
         index = self.indexOf(anchor)
@@ -11,5 +14,6 @@ def insert(self, el: QWidget, anchor: QWidget = None):
         el.setParent(self)
 
 
+@PySideRenderer.register_remove(QSplitter)
 def remove(self, el: QWidget):
     el.setParent(None)
