@@ -1,6 +1,10 @@
+from PySide6.QtWidgets import QStatusBar
+
+from ... import PySideRenderer
 from . import widget
 
 
+@PySideRenderer.register_insert(QStatusBar)
 def insert(self, el, anchor=None):
     # TODO: support for anchor
     # Support for anchor would be tough: there is no API to get
@@ -18,11 +22,13 @@ def insert(self, el, anchor=None):
     el.setParent(self)
 
 
+@PySideRenderer.register_remove(QStatusBar)
 def remove(self, el):
     self.removeWidget(el)
     el.setParent(None)
 
 
+@PySideRenderer.register_set_attr(QStatusBar)
 def set_attribute(self, attr, value):
     if attr == "text":
         if isinstance(value, tuple):
